// Package scripts contains scripts (pretty unexpected huh?) that
// facilitate testing and development. See the project readme for
// more information:
// https://github.com/smartystreets/goconvey/blob/master/README.md
package scripts
