/* Eric Meyer's Reset CSS v2.0 */
html,body,div,span,applet,object,iframe,h1,h2,h3,h4,h5,h6,p,blockquote,pre,a,abbr,acronym,address,big,cite,code,del,dfn,em,img,ins,kbd,q,s,samp,small,strike,strong,sub,sup,tt,var,b,u,i,center,dl,dt,dd,ol,ul,li,fieldset,form,label,legend,table,caption,tbody,tfoot,thead,tr,th,td,article,aside,canvas,details,embed,figure,figcaption,footer,header,hgroup,menu,nav,output,ruby,section,summary,time,mark,audio,video{border:0;font-size:100%;font:inherit;vertical-align:baseline;margin:0;padding:0}article,aside,details,figcaption,figure,footer,header,hgroup,menu,nav,section{display:block}body{line-height:1}ol,ul{list-style:none}blockquote,q{quotes:none}blockquote:before,blockquote:after,q:before,q:after{content:none}table{border-collapse:collapse;border-spacing:0}

::selection {
	background: #333;
	color: #FFF;
	text-shadow: none;
}

::-moz-selection {
	background: #333;
	color: #FFF;
	text-shadow: none;
}

html,
body {
	color: #333;
	font-family: 'Open Sans', sans-serif;
	height: 100%;
	min-height: 100%;
}

b {
	font-weight: bold;
}

i {
	font-style: italic;
}

small {
	font-size: 12px;
	line-height: 1.25em;
	color: #555;
}

hr {
	border: 0;
	height: 1px;
	background: #CCC;
}

a {
	color: #3771C8;
	font-weight: normal;
	text-decoration: none;
}

a:hover {
	color: #3581F6;
	text-decoration: underline;
}

code {
	font-family: 'Monaco', monospace;
	line-height: 1.5em;
	font-size: 10px;
	white-space: pre-wrap;
}

table {
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	box-sizing: border-box;

	width: 100%;
	border-collapse: collapse;
}

header {
	background: #F2F2F2;
	padding: .75% 0 .25%;
	border-bottom: 1px solid #555;
}

h1 {
	font: 33px 'Oswald', sans-serif;
	color: #000;
}

h1 a {
	color: inherit;
	text-decoration: none !important;
}

input, textarea {
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	box-sizing: border-box;

	outline: none;
	font: 14px 'Open Sans', 'Helvetica Neue', sans-serif;
	padding: 6px 10px;
}

textarea {
	padding: 10px;
	font: 300 18px/1.5em 'Open Sans', sans-serif;
	width: 100%;
}

ul, ol {
	margin: .75em 0;
}

ul li {
	list-style: disc;
	margin-left: 1.5em;
	margin-bottom: .5em;
}

#path {
	width: 100%;
	display: block;
	font-weight: 600;
	margin-top: 1.5%;
	padding-left: 2.5em;
}

.watching-eye {
	position: absolute;
	left: 1em;
	top: .95em;
	color: #BBB;
}


a.zen-close {
	font-size: 24px;
}

.menu {
	margin-top: 1.5%;
	text-align: center;
}

a.link-fa {
	color: #444;
	margin: 0px 10px 0;
	cursor: pointer;
}

a.link-fa:hover {
	color: #000;
	text-decoration: none;
}

a.link-fa.clr-red:hover {
	color: #FF0000 !important;
}

a.link-fa.disabled {
	color: #AAA;
	cursor: default;
}

#server-down {
	padding: 10px 0;
	background: #F7CACA;
	color: #CC0000;
	border-bottom: 1px solid #CC0000;
}



.overall {
	padding: 10px 0;
	box-shadow: inset 0 3px 5px -2px rgba(0, 0, 0, .5), inset 0 -3px 8px -3px rgba(0, 0, 0, .35);
	position: relative;
	z-index: 50;
	display: none;
}

.overall.ok {
	background: #5AA02C;
}

.overall.fail {
	background: #E79C07;
}

.overall.panic {
	background: #AA0000;
}

.overall.buildfail {
	background: #AAA;
}

.overall .status {
	float: left;
	margin-right: .75em;
	color: #FFF;
	font: bold 80px 'Georgia', serif;
	text-transform: uppercase;
}

.overall .go-up {
	display: none;
	margin-top: .3em;
	float: right;
	color: #FFF;
}

.overall.buildfail .status {
	font-size: 65px;
}

.overall .summary {
	float: left;
	font: 18px/1.25em monospace;
	color: rgba(255, 255, 255, .85);
	margin-top: .5em;
}








#results {
	display: none;
	margin-top: 1.5em;
}









.toggle {
	text-decoration: none !important;
	color: #666 !important;
}

.toggle-package-shortcuts,
.toggle-spacer {
	padding: 0 5px;
	display: inline-block;
}

.goto,
.toggle {
	text-decoration: none !important;
	color: #666 !important;
}

.goto:hover,
.toggle:hover {
	color: #000 !important;
}








.failure-shortcuts,
.panic-shortcuts,
.builds-shortcuts {
	margin-bottom: 1em;
}

.failure-shortcuts a,
.panic-shortcuts a,
.builds-shortcuts a {
	line-height: 1.5em;
}

.failure-shortcuts a {
	color: #E79C07;
}
.failure-shortcuts a:hover {
	color: #FFAD2B;
}
.panic-shortcuts a {
	color: #CC0000;
}
.panic-shortcuts a:hover {
	color: #FF0000;
}
.builds-shortcuts a {
	color: #666;
}
.builds-shortcuts a:hover {
	color: #AAA;
}

.package-shortcut {
	padding: .15em 0;
}

.testfunc-list {
	max-height: 200px;
	overflow-y: auto;
	padding-left: .75em;
	margin-top: .5em;
	display: none;
}

.testfunc-list a {
	display: block;
}









.failures,
.panics,
.builds {
	font-size: 12px;
	margin-bottom: 2em;
}

.failures .failure table,
.panics .panic table,
.builds .build table {
	margin-bottom: 1em;
}

.failures .failure table {
	border: 3px solid #E79C07;
}

.panics .panic table {
	border: 3px solid #CC0000;
}

.builds .build table {
	border: 3px solid #888;
}


.file,
.line,
.panic-error,
.stacktrace {
	font-family: 'Monaco', monospace;
	font-size: 14px;
}

.file {
	line-height: 1em;
	font-weight: bold;
	word-break: break-all;
}

.line {
	white-space: nowrap;
	text-align: right;
	vertical-align: middle;
	width: 100px;
}

.expected,
.actual,
.difference {
	width: 75px;
	text-align: left;
}


.failures .failure th,
.failures .failure td,
.panics .panic th,
.panics .panic td,
.builds .build th,
.builds .build td {
	border: 1px solid #CCC;
	padding: 5px;
}

.failures .failure th,
.panics .panic th {
	text-transform: uppercase;
}

.raw-output {
	padding: 0 !important;
}

.raw-output code {
	display: block;
	max-height: 200px;
	overflow-y: auto;
	font-size: 10px;
	line-height: 1.5em;
	word-break: break-all;
	background: #F0F0F0;
	padding: 5px;
}

.message {
	white-space: pre-wrap;
	word-break: break-all;
	font: 12px/1.5em 'Monaco', monospace;
	color: #666;
}



ins, .failures th.actual {
	background: #FFC6C6;
	text-decoration: none;
}

del, .failures th.expected {
	background: #C6FFC6;
}














.package-story {
	font: 14px 'Open Sans', sans-serif;
	line-height: 1.75em;
	width: 100%;
	margin-bottom: 3em;
}

.package-story th {
	text-align: left;
	padding: 10px;
	font-size: 16px;
	background: #EEE;
	line-height: 1.25em;
	word-wrap: break-word;
	font-weight: bold;
}

.package-story .package-top {
	position: relative;
	text-shadow: 1px 1px 1px #FFF;
	cursor: default;
}

.package-story td,
.package-story th {
	border: 1px solid #E0E0E0;
}

.package-story .ignore {
	float: right;
}

.package-story .package-header {
	position: relative;
	z-index: 1;
}

.package-story .pkg-summary {
	padding: 5px 10px;
	font-size: 12px;
	background: #F8F8F8;
}

.package-story .pkg-summary.inverse {
	background: #333;
	color: #FFF;	
}

.package-story .status {
	width: 5px;
	min-width: 5px;
}

.package-story .status.ok {
	background: #A6D37C;
}

.package-story .status.fail {
	background: #F7B336;
}

.package-story .status.panic {
	background: #FF8585;
}

.package-story .checks {
	white-space: nowrap;
	width: 50px;
	text-align: center;
	color: #5AA02C;
	font-weight: bold;
	border-right: 0;
	min-width: 25px;
}

.package-story .checks .ok {
	color: #5AA02C;
}

.package-story .checks .fail {
	color: #E79C07;
}

.package-story .checks .panic {
	color: #AA0000;
}

.package-story .checks .skip {
	color: #AAA;
}

.package-story .title {
	padding: .75em .5em;
	line-height: 1.25em;
	word-break: break-word;
}

.package-story .title small {
	white-space: pre-wrap;
}

.coverage {
	position: absolute;
	top: 0;
	left: 0;
	height: 100%;
	width: 5%;
	z-index: 0;
	border-bottom-right-radius: 25px;
	border-top-right-radius: 25px;
}

.depth-0 .title { padding-left: 1.5em !important; }
.depth-1 .title { padding-left: 3em !important; }
.depth-2 .title { padding-left: 4.5em !important; }
.depth-3 .title { padding-left: 6em !important; }
.depth-4 .title { padding-left: 7.5em !important; }
.depth-5 .title { padding-left: 9em !important; }
.depth-6 .title { padding-left: 10.5em !important; }
.depth-7 .title { padding-left: 11em !important; }








.sidebar {
	font: 12px 'Open Sans', sans-serif;
	margin-bottom: 2em;
	word-break: break-word;
}

#loading {
	text-align: center;
	text-transform: uppercase;
	font-size: 35px;
	font-weight: 300;
	color: #AAA;
	padding: 4em 0;
}


#footer {
	text-align: center;
	font-size: 12px;
	margin-top: 1em;
	margin-bottom: 4em;
	color: #888;
	line-height: 1.5em;
}















.text-center {
	text-align: center;
}

.clr-red {
	color: #CC0000 !important;
}

.clr-maroon {
	color: #7F1502 !important;
}










#loader {
	position: fixed;
	top: 20px;
	right: 20px;
	z-index: 99;
	width: 24px;
}


.zen {
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	box-sizing: border-box;


	display: none;
	position: fixed;
	overflow: scroll;
	height: 100%;
	width: 100%;
	padding: 3%;
	top: 0;
	left: 0;
	background: rgba(255, 255, 255, .97);
	z-index: 51;
}

.zen h1 {
	font-size: 56px;
	text-align: center;
}

.zen p {
	margin: 1em 0;
	line-height: 1.5em;
	font-size: 20px;
	font-weight: 300;
}

.zen-close {
	position: fixed;
	top: 3%;
	right: 5%;
	z-index: 52;
	padding: .5em;
}


#gen-input {
	margin-bottom: 2em;
	resize: none;
}

#gen-output {
	white-space: pre;
	font-size: 16px;
	font-family: 'Monaco', monospace;
	padding: 10px;
	color: #333;
	background: #F0F0F0;
	display: block;
	border: 0;
	resize: vertical;
	height: 300px;
}


.stuck {
	position: fixed;
	top: 0;
	width: 100%;
	background: white;
	z-index: 48;
}

.stuck .overall {
	box-shadow: 0px 1px 3px #333;
}

.stuck .overall .status {
	font-size: 22px !important;
}

.stuck .overall .summary {
	display: none;
}

.stuck .overall .go-up {
	display: block;
}

















@media (max-width: 1024px) {
	h1 {
		text-align: center;
	}

	.watching-eye {
		top: .5em;
	}

	.package-story th {
		font-size: 12px;
	}
}

@media (max-width: 600px) {
	.overall .status {
		font-size: 60px;
		margin-right: .4em;
	}

	.overall .summary {
		font-size: 14px;
	}

	.package-story th,
	.package-story .title {
		word-break: break-word;
		font-size: 12px;
	}

	.package-story .title {
		padding-top: 5px;
		padding-bottom: 5px;
	}
}