// Package execution contains internal functionality for executing
// specifications in an isolated fashion as well as calling reporting
// hooks at important points in the process. Although this package has
// exported names is not intended for public consumption. See the
// examples package for how to use this project.
package execution
